
// ****************************************
// IMPORT IMAGE
// ****************************************


// VICE-USAGE
// - drag&drop
// - F10
// - cursors (0-10)

// KICK ASSEMLBER
// http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html

// CONVERT PICS TO C64 COLOR PALETTE
// https://www.micheldebree.nl/retropixels/

// GRAPHICS PAIN
// https://mcdraw.xyz

// MODES:
// - CURSOR MODE - ASCII 
// - GRAPHIC MODE - ALSO BLOCKS
// - HIGHRES: 320x200 
// - LOWRES: 160x200 - MULTICOLOR 

// DEMO EXAMPLE
// https://github.com/celso/c64/blob/master/card.asm

// RANDOM
// rndGen(%1111);


// zone Farben
// *******************************************************************************
// *** Farben                                                                  ***
// *******************************************************************************
.var COLOR_BLACK         = $00           // schwarz
.var COLOR_WHITE         = $01           // weiß
.var COLOR_RED           = $02           // rot
.var COLOR_CYAN          = $03           // türkis
.var COLOR_PURPLE        = $04           // lila
.var COLOR_GREEN         = $05           // grün
.var COLOR_BLUE          = $06           // blau
.var COLOR_YELLOW        = $07           // gelb
.var COLOR_ORANGE        = $08           // orange
.var COLOR_BROWN         = $09            // braun
.var COLOR_PINK          = $0a           // rosa
.var COLOR_DARKGREY      = $0b           // dunkelgrau
.var COLOR_GREY          = $0c           // grau
.var COLOR_LIGHTGREEN    = $0d           // hellgrün
.var COLOR_LIGHTBLUE     = $0e           // hellblau
.var COLOR_LIGHTGREY     = $0f           // hellgrau

// zone Sprites
// *******************************************************************************
// *** Die VIC II Sprite Register  -  ANFANG                                   ***
// *******************************************************************************
.var SPRITE0X            = $d000         // (00) X-Position von Sprite 0
.var SPRITE0Y            = $d001         // (01) Y-Position von Sprite 0
.var SPRITE1X            = $d002         // (02) X-Position von Sprite 1
.var SPRITE1Y            = $d003         // (03) Y-Position von Sprite 1
.var SPRITE2X            = $d004         // (04) X-Position von Sprite 2
.var SPRITE2Y            = $d005         // (05) Y-Position von Sprite 2
.var SPRITE3X            = $d006         // (06) X-Position von Sprite 3
.var SPRITE3Y            = $d007         // (07) Y-Position von Sprite 3
.var SPRITE4X            = $d008         // (08) X-Position von Sprite 4
.var SPRITE4Y            = $d009         // (09) Y-Position von Sprite 4
.var SPRITE5X            = $d00a         // (10) X-Position von Sprite 5
.var SPRITE5Y            = $d00b         // (11) Y-Position von Sprite 5
.var SPRITE6X            = $d00c         // (12) X-Position von Sprite 6
.var SPRITE6Y            = $d00d         // (13) Y-Position von Sprite 6
.var SPRITE7X            = $d00e         // (14) X-Position von Sprite 7
.var SPRITE7Y            = $d00f         // (15) Y-Position von Sprite 7
.var SPRITESMAXX         = $d010         // (16) Höhstes BIT der jeweiligen X-Position
                                    //         da der BS 320 Punkte breit ist reicht
                                    //         ein BYTE für die X-Position nicht aus// 
                                    //         Daher wird hier das 9. Bit der X-Pos
                                    //         gespeichert. BIT-Nr. (0-7) = Sprite-Nr.
.var SPRITEACTIVE        = $d015         // (21) Bestimmt welche Sprites sichtbar sind
                                    //         Bit-Nr. = Sprite-Nr.
.var SPRITEDOUBLEHEIGHT  = $d017         // (23) Doppelte Höhe der Sprites
                                    //         Bit-Nr. = Sprite-Nr.
.var SPRITEDEEP          = $d01B         // (27) Legt fest ob ein Sprite vor oder hinter
                                    //         dem Hintergrund erscheinen soll.
                                    //         Bit = 1: Hintergrund vor dem Sprite
                                    //         Bit-Nr. = Sprite-Nr.
.var SPRITEMULTICOLOR    = $d01c         // (28) Bit = 1: MultiColor Sprite 
                                    //         Bit-Nr. = Sprite-Nr.
.var SPRITEDOUBLEWIDTH   = $d01d         // (29) Bit = 1: Doppelte Breite des Sprites
                                    //         Bit-Nr. = Sprite-Nr.
.var SPRITEMULTICOLOR0   = $d025         // (37) Spritefarbe 0 im Multicolormodus
.var SPRITEMULTICOLOR1   = $d026         // (38) Spritefarbe 1 im Multicolormodus
.var SPRITE0COLOR        = $d027         // (39) Farbe von Sprite 0
.var SPRITE1COLOR        = $d028         // (40) Farbe von Sprite 1
.var SPRITE2COLOR        = $d029         // (41) Farbe von Sprite 2
.var SPRITE3COLOR        = $d02a         // (42) Farbe von Sprite 3
.var SPRITE4COLOR        = $d02b         // (43) Farbe von Sprite 4
.var SPRITE5COLOR        = $d02c         // (44) Farbe von Sprite 5
.var SPRITE6COLOR        = $d02d         // (45) Farbe von Sprite 6
.var SPRITE7COLOR        = $d02e         // (46) Farbe von Sprite 7

.var SPRITE0DATA         = $07f8         // Sprite-Pointer für die Sprite-0-Daten
// *******************************************************************************
// *** Die VIC II Sprite Register  -  ENDE                                     ***
// *******************************************************************************

// zone VIC
.var VICBASE             = $d000         // (RG) = Register-Nr. dezimal
.var VICRASTERROWPOS     = $d012         // (18) Aktuelle Rasterzeile lesen oder setzen
.var VICBORDERCOLOR      = $d020         // (32) Bildschirmrandfarbe
.var VICBACKGROUNDCOLOR  = $d021         // (33) Hintergrundfarbe

// zone CIA
.var CIA1_A              = $dc00         // Adresse des CIA1-A
.var CIA1_B              = $dc01         // Adresse des CIA1-B
.var CIA1_CLOCK_TENTH    = $dc08         // Adresse zehntel Sekunden (Uhrzeit) im CIA1
.var CIA1_CLOCK_SECONDS  = $dc09         // Adresse Sekunden (Uhrzeit) im CIA1

// zone SID
// *******************************************************************************
// *** Die SID Register  -  ANFANG                                             ***
// *******************************************************************************
.var SID_BASE            = $d400         // (Nr) Basisadresse des SID_BASE
.var SID_VOICE1_FREQ_MSB = $d401         // (01) 1. Stimme: High-Byte der Frequenz
.var SID_VOICE1_WAVEFORM = $d404         // (04) 1. Stimme: Wellenform
.var SID_SUSTAIN_RELEASE = $d406         // (06) BIT 7-4: Anschlag (sustain)
                                    //          3-0: Abschwellen (release)
.var SID_VOLUME          = $d418         // (24) BIT   7: 3. Stimme Filtermodus AUS
                                    //            6:   -''-    Hochpass
                                    //            5:   -''-    Bandpass
                                    //            4:   -''-    Tiefpass
                                    //          3-0: Lautstärke

.var SID_WAVE_TRIANGLE   = $11           // Wellenform: Dreieck
.var SID_WAVE_SAWTOOTH   = $21           // Wellenform: Sägezahn
.var SID_WAVE_PULSE      = $41           // Wellenform: Rechteck
.var SID_WAVE_WHITENOISE = $81           // Wellenform: Rauschen
// *******************************************************************************
// *** Die SID Register  -  ENDE                                               ***
// *******************************************************************************

// zone Zeropage
.var ZP_SCREENRAMADR     = $02           // Zeichen-Quelle fürs BS-Kopieren (drawscreen:)
.var ZP_COLORRAMADR      = $04           // Farb-Quelle fürs BS-Kopieren    (drawscreen:)
.var ZP_SCREENTODRAW     = $06           // Zeichen-Ziel fürs BS-Kopieren   (drawscreen:)
.var ZP_COLORTODRAW      = $08           // Farb-Ziel fürs BS-Kopieren      (drawscreen:)
.var ZP_LOOPHELPER       = $f8           // Die letzten 8 BYTES für Schleifen reservieren

// zone Kernal
.var JT_GETIN            = $ffe4         // Kernalfunktion: Zeichen lesen

// zone Generic
.var SCREENRAMADR        = $0400         // Beginn des Bildschirmspeichers
.var COLORRAMADR         = $d800         // Beginn des Farbspeichers
.var INPUT_NONE          = $00           // Es wurde noch kein Port gewählt
.var INPUT_JOY1          = $01           // Joystick in Port-1
.var INPUT_JOY2          = $02           // oder 2
.var JOY_UP              = %00000001     // Joystick rauf
.var JOY_DOWN            = %00000010     // Joystick runter
.var JOY_LEFT            = %00000100     // Joystick links
.var JOY_RIGHT           = %00001000     // Joystick rechts
.var JOY_FIRE            = %00010000     // Joystick FEUER// 


// ---------------
// MINIMAL
// ---------------
.var  BORDER_COLOR   = $D020
.var  BACKGROUND_COLOR    = $D021
.var  FOREGROUND_COLOR = $E536

.var  SPRITE_0_X_POSITION = $D000
.var  SPRITE_0_Y_POSITION = $D001
.var  SPRITE_0_POINTER    = $0400 + $03F8 // Last 8 Bytes of Screen RAM
.var  SPRITE_0_DATA       = $0340         // Block 13, 13*64=>832 => $0340
.var SPRITE_UPPER_X      = $D010
.var  SPRITE_0_COLOR      = $D027 

// COLORS
// https://www.c64-wiki.com/wiki/Color

// SPRITE EDITOR online editor: https://www.spritemate.com
// multicolor: https://beta.spritemate.com

// look at: https://www.magic64knight.com/2020/01/sample-sprite-code.html
// 

// IDEAS: 
// Strategy: Strategy game with frame. defend frame coming into the frame and destroying it ... > change color > change strategy
//           not all colors are good. 

// Demo: Use loading input on screen ... (someone did this of course before, i am sure) title could be content

*=$0801   //  Starting Address BASIC + 1 =&gt//  SYS 2049
 
  .byte $0C,$08,$40,$00,$9E,$20,$32,$30,$36,$32,$00,$00,$00 //  BASIC CODE: 1024 SYS 2062



// --------------------------------------------------
// LOAD MUSIC
// --------------------------------------------------
.var music = LoadSid("Nightshift.sid")    //<- Here we load the sid file


// --------------------------------------------------
// INIT 
// --------------------------------------------------

init:

          // CLEAR SCREEN
          jsr clear_screen


          // LOAD MUSIC NOW
          // http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html

          lda #$00
          sta $d020
          sta $d021
          ldx #0
          ldy #0
          lda #music.startSong-1            //<- Here we get the startsong and init address from the sid file
          jsr music.init  
          sei
          lda #<irq1
          sta $0314
          lda #>irq1
          sta $0315
          lda #$1b
          sta $d011
          lda #$80
          sta $d012
          lda #$7f
          sta $dc0d
          sta $dd0d
          lda #$81
          sta $d01a
          lda $dc0d
          lda $dd0d
          asl $d019
          cli

          // DEC TO HEX
          // https://www.rapidtables.com/convert/number/decimal-to-hex.html?x=214
         
          jsr disable_inputbuffers

          // ----------------
          // BACK & FOREGROUND COLORS
          // ----------------
          // background, frame color
          lda #$04     
          sta BORDER_COLOR      
          sta BACKGROUND_COLOR  
          // set FOREGROUND_COLOR (COLOR CAN BE DIFFERENT IN EACH LINE!)
          lda #$01 
          jsr FOREGROUND_COLOR


          // ----------------
          // PRINT
          // ----------------
          // zeichensatz: https://www.c64-wiki.de/wiki/Zeichen

          // print - show a character
          lda #$d6                //  load character '_' into register A
          jsr $ffd2               //  print character to screen

          // print at // ... 
          // https://medium.com/@RetroAsmDev/c-64-part-4-ascii-art-a4319784d26


          // KEY
          .var SCNKEY     = $ff9f   // scan keyboard - kernal routine
          .var GETIN      = $ffe4   // read keyboard buffer - kernal routine


          // ----------------
          // SPRITE INIT
          // ----------------


        // all multicolor
          .var spriteMulticolour = $d01c
          lda #$00
          sta spriteMulticolour


          lda #$0D              // using block 13 for sprite0
          sta SPRITE_0_POINTER  // set block 13 as target address for Data of Sprite0
          lda #$01              // enable...
          sta $D015             // ...Sprite 0 => %0000 0001 (all sprites off except Sprite 0)

          lda #$00     // load black color code into A
          sta SPRITE_0_COLOR    // make Sprite0 completely black
          
          // Reset Sprite Data
          ldx #$00    // init x
          lda #$00    // init a

        clean:
          sta SPRITE_0_DATA,x   // write 0 into sprite data at x
          inx                   // increment x
          cpx #$3F              // is x <= 63?
          bne clean             // if yes, goto clean

          //  Build the Sprite
          ldx #$00              //  init x
        build:
        //  lda sprite_data, x           //  load data at x
          lda racket, x           //  load data at x
          sta SPRITE_0_DATA,x   //  write into sprite data at x
          inx                   //  increment x
          cpx #$3F              //  is x <= 63?
          bne build             //  if yes, goto build
          
          //  Set Start Location of Sprite 0
          ldx #$64                  //  initial x position = 100
          ldy #$46                  //  initial y position =  70
          stx SPRITE_0_X_POSITION   //  move sprite 0 to x position
          sty SPRITE_0_Y_POSITION   //  move sprite 0 to y position
          // ----------------

        // SPRITE INIT FROM EDITOR

        lda #$04 // sprite multicolor 1
        sta $D025
        lda #$06 // sprite multicolor 2
        sta $D026

        // -------------------
        // SCREEN MODE
        // -------------------
        /*
        // Bitmap Mode On
        lda #$3b
        sta $d011
        // MultiColor On
        lda #$d8
        sta $d016
        */

        // ---------------------------
        // SCREENMODE: ASCII
        // ---------------------------
        // WRITE INTO SCREEN MEMORy
        // ASCII: 40x25 > 1000bytes > $0400-$07F7
        // - example: https://dustlayer.com/c64-coding-tutorials/2013/4/8/episode-2-2-writing-to-the-c64-screen
        

        // all the same
        ldx #$00
        ldy #$00
clear:    
         lda #$32     // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         lda #$08     // set foreground to black in Color Ram 
         sta $d800,x  
         sta $d900,x
         sta $da00,x
         sta $dae8,x
         inx           // increment X
         bne clear    

         // change colors on and on

        ldx #$00
        ldy #$00
        lda #$00
clearcol:    
         lda #$42     // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         // lda #$08     // set foreground to black in Color Ram 
         iny
         tya
         sta $d800,x  
         iny
         tya
         sta $d900,x
         iny
         tya
         sta $da00,x
         // iny
         // tya
         sta $dae8,x
         inx           // increment X
         bne clearcol    
        


// --------------------------------------------------
// LOOP 
// --------------------------------------------------

loop:
  
  
     jsr vsync       // vsync


     /*

        // ASCII-SCREEN
        // PRINTING 

        // https://dustlayer.com/c64-coding-tutorials/2013/4/8/episode-2-4-reading-from-a-data-table

        ldx #$00
        ldy #$00

        ldy counter
        inc counter

        lda #$00        
clearcolhere:    
         lda #$42     // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         // lda #$08     // set foreground to black in Color Ram 
         iny
         tya
         sta $d800,x  
         iny
         tya
         sta $d900,x
         iny
         tya
         sta $da00,x
         // iny
         // tya
         sta $dae8,x
         inx           // increment X
         bne clearcolhere    
        
        */  





            // ---------
            // keys     ascii? > other in charset! 
            // ---------
            // input keys: https://codebase64.org/doku.php?id=base:scanning_the_keyboard_the_correct_and_non_kernal_way
            // get all keys https://codebase64.org/doku.php?id=base:scanning_the_keyboard_the_correct_and_non_kernal_way 

            // https://dustlayer.com/c64-coding-tutorials/2013/5/24/episode-3-6-custom-character-sets-hello-charpad


// > check keys https://dustlayer.com/c64-coding-tutorials/2013/5/24/episode-3-5-taking-command-of-the-ship-controls


            jsr SCNKEY
            jsr GETIN

            // check keys

            // A                 
            cmp $1f
            bne notkey_a

            lda $d6               
            jsr $ffd2              
notkey_a:        

            cmp #13             // return
            bne notkey_return
            lda $03                
            jsr $ffd2 
notkey_return:                 

            // SPACE
            cmp #$20
            bne nokey
            // jmp exit

            lda #05     
            sta BORDER_COLOR      
            sta BACKGROUND_COLOR  

            rts  // exit ... 
nokey:  

            // ------------------
            // joystick  
            // ------------------
            // joystick : https://retro-programming.com/how-to-program-joystick-input-handling-on-a-c64/

            lda #$08                // mask left movement (8 == bit 4 == %0000 1000)
            bit $DC01               // bitwise "and" with joystick port 1
            bne joystick_notright   // if not active (==1), go to .input_up_check
            lda #$32
            jsr $ffd2   // print screen ... 
            inc SPRITE_0_X_POSITION
            inc SPRITE_0_X_POSITION
joystick_notright:
            
            lda #$01              // mask left movement (8 == bit 4 == %0000 1000)
            bit $DC01               // bitwise "and" with joystick port 1
            bne joystick_notleft   // if not active (==1), go to .input_up_check
            lda #$33
            jsr $ffd2   // print screen ... 
            dec SPRITE_0_X_POSITION
            dec SPRITE_0_X_POSITION
            // dec #(SPRITE_0_X_POSITION-1)
joystick_notleft:

            
            // ------------------
            // sprite ...  
            // ------------------
            // https://retro-programming.com/how-to-move-sprites-in-c64-assembly/
            // https://www.c64-wiki.de/wiki/Sprite




    
            // ------------------
            // blinken ...  
            // ------------------
            /*
            inc BACKGROUND_COLOR
            inc BORDER_COLOR
            inc FOREGROUND_COLOR
            */

            


            // restart
            jmp loop      // loopy loop



exit: 

  // back to BASIC

// --------------------------------------------------
// SUBROUTINES 
// --------------------------------------------------
clear_screen:
    jsr $E544       
    rts

disable_inputbuffers:
    lda #1
    sta $0289  //  disable keyboard buffer
    lda #127
    sta $028a  // disable key repeat
    rts

vsync:
      lda #$FF    // load 255 into A
      cmp $D012    //look if current rasterline equals 255
      bne vsync     // if no, goto wait
      rts  


    // --------------------------------------------------
    // MUSIC PLAY
    // --------------------------------------------------
    irq1:
            asl $d019
            inc $d020
            jsr music.play 
            dec $d020
            pla
            tay
            pla
            tax
            pla
            rti

// --------------------------------------------------
// DATA 
// --------------------------------------------------
// VISUALS
// 
// online editor: https://www.spritemate.com
// multicolor: https://beta.spritemate.com

// sprite data
spritedata:

// 1 sprites generated with spritemate on 26.3.2023, 22:19:51
// Byte 64 of each sprite contains multicolor (high nibble) & color (low nibble) information


// sprite 0 / singlecolor / color: $00
sprite_0_mono:
.byte $40,$00,$00,$5f,$ff,$c0,$5f,$ff
.byte $f8,$58,$00,$c8,$5f,$80,$c9,$59
.byte $82,$fd,$59,$82,$fd,$1f,$82,$0d
.byte $1f,$83,$ed,$18,$00,$0d,$5f,$80
.byte $0c,$58,$00,$0e,$58,$00,$0e,$58
.byte $00,$0e,$19,$8c,$0c,$18,$d8,$0c
.byte $18,$70,$ec,$1f,$ff,$fc,$1f,$ff
.byte $fc,$1c,$00,$1c,$00,$00,$00,$00



racket:
.byte $00,$00,$00,$00,$00,$00,$03,$70
.byte $00,$0d,$9c,$00,$36,$a4,$3c,$36
.byte $e4,$3c,$36,$e4,$00,$35,$a4,$00
.byte $3d,$94,$00,$0d,$5c,$00,$0f,$70
.byte $00,$03,$c0,$f0,$00,$03,$ac,$00
.byte $0d,$7b,$00,$0f,$7b,$00,$0f,$db
.byte $00,$03,$fc,$00,$00,$f0,$00,$00
.byte $00,$00,$00,$00,$00,$00,$00,$81


// counter
counter:
  .byte $00


/*
  
OBJECTS ... 

*/

object_num: .byte $08

.var _x = 0
.var _y = 0
.var _w = 0
.var _h = 0
.var _s = 0
.var _a = 0
.var _st = 0
.var _sts = 0
.var _arg = 0

objects: 
  //    2 x      , 2y     , sizex, sizey, stat
  //    x [0,1]   y [2,3]   width[4] , height[5], spr[6], anim[7], stat[8], statsub[9], arg[10]

  // object  
  .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00



// MUSIC

  *=music.location "Music"
  .fill music.size, music.getData(i)        // <- Here we put the music in memory

// IMAGE

// *=$2800
// *=$4000

splashscreen:

.var logo = LoadPicture("image.png")
.fill $800, logo.getSinglecolorByte((i>>3)&$1f, (i&7) | (i>>8)<<3)

