// ****************************************
// GAME TEMPLATE
// IN KICK ASSEMBLER
// EXPERIMENTAL ARCHEOLOGY
// CONFOEDERATIO LUDENS
// 
// ****************************************

// PROCESSOR: 6502 :-( (Why not the original 680x? yes to expensive!)
// VIC-II: Graphic chip
// SID: Programmable sound chip

// EMULATION: VICE-USAGE
// - drag&drop
// - F10
// - cursors (0-10)

// KICK ASSEMLBER
// http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html

/*

    # ASSEMBLER-HINTS

    160x200 in multicolor 

    only 8bit ... > problems with sprite x etc .-(

    txa - transfer x to a
    tya - transfer a to x

    / indexing
    sta xyz,4 // store in acummlator-adresse xyz[4]
    lda xyz,5 // load into accumulator xyz[4]

    cmp #20,x
    bne bxt // not ...

    bxt

*/

// MODES:
// - CURSOR MODE - ASCII 
// - GRAPHIC MODE - ALSO BLOCKS
// - HIGHRES: 320x200 
// - LOWRES: 160x200 - MULTICOLOR 

// GRAPHICS PAINT
// https://mcdraw.xyz

// CONVERT PICS TO C64 COLOR PALETTE
// https://www.micheldebree.nl/retropixels/

// SPRITE PAINT
// 

// HELPERS
// DEC TO HEX
// https://www.rapidtables.com/convert/number/decimal-to-hex.html?x=214


// DEMO EXAMPLE
// https://github.com/celso/c64/blob/master/card.asm (including sprite management etc)

// RANDOM
// rndGen(%1111);
// sta $d027,x  // into x 



// MUSIC

// playing SID music has 3 parts:
// 0. load music
// 1. you use use Kick Assembler's LoadSid to import the .sid file
// 2. you call music.init
// 3. you call music.play in every raster interrupt
// see http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html and the README.md

// ----------------------
// MINIMAL VARS/CONST
// ----------------------

.var  BORDER_COLOR   = $D020
.var  BACKGROUND_COLOR    = $D021
.var  FOREGROUND_COLOR = $E536

.var  SPRITE_0_X_POSITION = $D000
.var  SPRITE_0_Y_POSITION = $D001
.var  SPRITE_0_POINTER    = $0400 + $03F8 // Last 8 Bytes of Screen RAM
.var  SPRITE_0_DATA       = $0340         // Block 13, 13*64=>832 => $0340
.var SPRITE_UPPER_X      = $D010
.var  SPRITE_0_COLOR      = $D027 

 .var SCNKEY     = $ff9f   // scan keyboard - kernal routine
  .var GETIN      = $ffe4   // read keyboard buffer - kernal routine

// --------------------------------------------
// LOAD DATA
// --------------------------------------------

// Music (<3k!)
// Attention: Reload Emulators to test other music!!

.var music = LoadSid("music7.sid")

// Graphics
// https://mcdraw.xyz
.var picture = LoadBinary("image.koa", BF_KOALA)
// .var picture = LoadBinary("minimalpicture.kla", BF_KOALA) // less loading


// --------------------------------------------------
// INIT
// --------------------------------------------------

*=$0801 "Basic Program"
BasicUpstart($0810)

*=$0810 "Program"

// --------------------------------------------------
// INIT 
// --------------------------------------------------


    // MULTIMEDIA

    // music: init 1
    lda #music.startSong-1 // set up the music
    jsr music.init

    // setup: raster music
    sei
      lda #<irq1
      sta $0314
      lda #>irq1
      sta $0315
      lda #$1b
      sta $d011
      lda #$80
      sta $d012
      lda #$7f
      sta $dc0d
      sta $dd0d
      lda #$81
      sta $d01a
      lda $dc0d
      lda $dd0d
      asl $d019
      cli


    // CLEAR SCREEN
    jsr clear_screen

     // disable input buffers     
     jsr disable_inputbuffers



// ------------------------------------
// asciiloop (SHOWS 'ASCII' MODE)
// ------------------------------------
// of course: it is no ascii!!!!
// different format like zx81 etc
 
jmp noascii // disable 'ASCII'    

    // background, frame color
      lda #$04     
      sta BORDER_COLOR      
      lda #$07
      sta BACKGROUND_COLOR  
      // set FOREGROUND_COLOR (COLOR CAN BE DIFFERENT IN EACH LINE!)
      lda #$09
      sta FOREGROUND_COLOR


      // ----------------
      // ASCII / PRINT
      // ----------------
      
      // charset: https://www.c64-wiki.de/wiki/Zeichen

      // print - show a character
      lda #$d6                //  load character '_' into register A
      jsr $ffd2               //  print character to screen
      // print at // ... 
      // https://medium.com/@RetroAsmDev/c-64-part-4-ascii-art-a4319784d26
      
  

        
        // WRITE INTO SCREEN MEMORY

        // ASCII: 40x25 > 1000bytes > $0400-$07F7
        // - example: https://dustlayer.com/c64-coding-tutorials/2013/4/8/episode-2-2-writing-to-the-c64-screen
        
        // FILL ALL WITH CHARS

        // all the same
        ldx #$00
        ldy #$00
clear:    
         lda #$40    // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         lda #$04     // set foreground to black in Color Ram 
         sta $d800,x  
         sta $d900,x
         sta $da00,x
         sta $dae8,x
         inx           // increment X
         bne clear    


asciiloop: 
        jsr vsync // wait for 'vsync'

        jsr SCNKEY
        jsr GETIN

        // SPACE TO FINISH
        cmp #$20
        bne noinfokey
            // lda #05     
            // sta BORDER_COLOR      
            // sta BACKGROUND_COLOR  
            
            lda #$d6                //  load character '_' into register A
            jsr $ffd2               //  print character to screen
              
            jmp noascii
noinfokey:

        // PRINTING 
        // https://dustlayer.com/c64-coding-tutorials/2013/4/8/episode-2-4-reading-from-a-data-table

        ldx #$00
        ldy #$00

        ldy counter
        inc counter

        lda #$00        
clearcolhere:    
         lda #$42     // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         // lda #$08     // set foreground to black in Color Ram 
         iny
         tya
         sta $d800,x  
         iny
         tya
         sta $d900,x
         iny
         tya
         sta $da00,x
         // iny
         // tya
         sta $dae8,x
         inx           // increment X
         bne clearcolhere   

        
        // ------------------
        // blink  
        // ------------------
    
        /*
        inc BACKGROUND_COLOR
        inc BORDER_COLOR
        inc FOREGROUND_COLOR
        */

        jmp asciiloop

noascii:
        


      // ----------------
      // GRAPHIC MODE
      // ----------------
 

      // copy picture 

        lda #$38
        sta $d018
        lda #$d8
        sta $d016
        lda #$3b
        sta $d011
        lda #0
        sta $d020
        lda #picture.getBackgroundColor()
        sta $d021
        ldx #0
!loop:
        .for (var i=0; i<4; i++) {
           lda colorRam+i*$100,x
           sta $d800+i*$100,x
        }
        inx
        bne !loop-
        jmp *



// --------------------------------------------------
// LOOP 
// --------------------------------------------------

loop:
  
  
           jsr SCNKEY
           jsr GETIN

              
            // SPACE TO FINISH
            cmp #$20
            bne nokey
            // jmp exit
            lda #05     
            sta BORDER_COLOR      
            sta BACKGROUND_COLOR  
            rts  // exit ... 
nokey:  

   
 
            // restart
            jmp loop      // loopy loop

            // ------------------ 


exit: 
            jsr clear_screen
            rts

            // back to BASIC

// --------------------------------------------------
// DATA LOADING 
// --------------------------------------------------

*=$0c00;            .fill picture.getScreenRamSize(), picture.getScreenRam(i)
*=$1c00; colorRam:  .fill picture.getColorRamSize(), picture.getColorRam(i)
*=$2000;            .fill picture.getBitmapSize(), picture.getBitmap(i)


* = music.location "Music"
.fill music.size, music.getData(i)


// VARIABLES
// counter
counter:
  .byte $00

// --------------------------------------------------
// SUBROUTINES 
// --------------------------------------------------
clear_screen:
    jsr $E544       
    rts

disable_inputbuffers:
    lda #1
    sta $0289  //  disable keyboard buffer
    lda #127
    sta $028a  // disable key repeat
    rts

vsync:
      lda #$FF    // load 255 into A
      cmp $D012    //look if current rasterline equals 255
      bne vsync     // if no, goto wait
      rts  


// --------------------------------------------------
// INTERRUPTS 
// --------------------------------------------------
irq1:
    asl $d019
    

    // inc $d020 - act. volume 
    jsr music.play 
    // dec $d020
    pla
    tay
    pla
    tax
    pla
    rti


// --------------------------------------------------
// MACROS 
// --------------------------------------------------
.macro rndGen(mask) {
    // see http://sta.c64.org/cbm64mem.html
    // first we read the current raster line (0-255)
    lda $d012
    // then we xor it with timer A (low byte)
    // see https://www.c64-wiki.com/wiki/CIA
    eor $dc04
    // then we subtract it with timer A (high byte)
    sbc $dc05
    // finally we mask it so we can have a number between 0 and bits^2
    and #mask
}




