// ****************************************
// GAME TEMPLATE
// IN KICK ASSEMBLER
// EXPERIMENTAL ARCHEOLOGY
// CONFOEDERATIO LUDENS
// 
// ****************************************

// COMPILING (FOR FASTER DEV)
.var loadPicture = true // > by hand ! search for loadPicture
.var loadMusic = true

// CONTROL 1-10 as joystick input1Ω

// PROCESSOR: 6502 :-( (Why not the original 680x? yes to expensive!)
// VIC-II: Graphic chip
// SID: Programmable sound chip

// EMULATION: VICE-USAGE
// - drag&drop
// - F10
// - cursors (0-10)

// KICK ASSEMLBER
// ASSEMBLER WITH SCRIPTING FUNCTIONS
// http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html

/*

    # ASSEMBLER-HINTS

    160x200 in multicolor 

    only 8bit ... > problems with sprite x etc .-(

    txa - transfer x to a
    tya - transfer a to x

    / indexing
    sta xyz,4 // store in acummlator-adresse xyz[4]
    lda xyz,5 // load into accumulator xyz[4]

    cmp #20,x
    bne bxt // not ...

    bxt

*/

// VIDEO RAMS ADDRESSES
// $0000-$3fff, $4000-$7fff, $8000-$bfff, $c000-$ffff
// ... 


// MODES:
// - CURSOR MODE - ASCII 
//   - start AT $4000 (SCREENRAM)

// - GRAPHIC MODE - ALSO BLOCKS (COLORRAM)
//   - STARTS AT $D800

// - HIGHRES: 320x200 
// - LOWRES: 160x200 - MULTICOLOR 

// GRAPHICS PAINT
// https://mcdraw.xyz

// CONVERT PICS TO C64 COLOR PALETTE
// https://www.micheldebree.nl/retropixels/

// SPRITE PAINT (Multicolor)
// multicolor: https://beta.spritemate.com


// HELPERS
// DEC TO HEX
// https://www.rapidtables.com/convert/number/decimal-to-hex.html?x=214


// DEMO EXAMPLE
// https://github.com/celso/c64/blob/master/card.asm (including sprite management etc)
// GOOD FOR STARTING (this is a mix from this and other tutorials)

// RANDOM
// rndGen(%1111);
// sta $d027,x  // into x 



// MUSIC

// playing SID music has 3 parts:
// 0. load music
// 1. you use use Kick Assembler's LoadSid to import the .sid file
// 2. you call music.init
// 3. you call music.play in every raster interrupt
// see http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html and the README.md

// ----------------------
// MINIMAL VARS/CONST
// ----------------------

.var  BORDER_COLOR   = $D020
.var  BACKGROUND_COLOR    = $D021
.var  FOREGROUND_COLOR = $E536

.var  SPRITE_0_X_POSITION = $D000
.var  SPRITE_0_Y_POSITION = $D001
.var  SPRITE_0_POINTER    = $0400 + $03F8 // Last 8 Bytes of Screen RAM
.var  SPRITE_0_DATA       = $0340         // Block 13, 13*64=>832 => $0340
.var SPRITE_UPPER_X      = $D010
.var  SPRITE_0_COLOR      = $D027 

 .var SCNKEY     = $ff9f   // scan keyboard - kernal routine
 .var GETIN      = $ffe4   // read keyboard buffer - kernal routine

            // ----------------------
            // MAX VARIABLES
            // ----------------------


            // *******************************************************************************
            // *** Farben                                                                  ***
            // *******************************************************************************
            .var COLOR_BLACK         = $00           // schwarz
            .var COLOR_WHITE         = $01           // weiß
            .var COLOR_RED           = $02           // rot
            .var COLOR_CYAN          = $03           // türkis
            .var COLOR_PURPLE        = $04           // lila
            .var COLOR_GREEN         = $05           // grün
            .var COLOR_BLUE          = $06           // blau
            .var COLOR_YELLOW        = $07           // gelb
            .var COLOR_ORANGE        = $08           // orange
            .var COLOR_BROWN         = $09            // braun
            .var COLOR_PINK          = $0a           // rosa
            .var COLOR_DARKGREY      = $0b           // dunkelgrau
            .var COLOR_GREY          = $0c           // grau
            .var COLOR_LIGHTGREEN    = $0d           // hellgrün
            .var COLOR_LIGHTBLUE     = $0e           // hellblau
            .var COLOR_LIGHTGREY     = $0f           // hellgrau

            // zone Sprites
            // *******************************************************************************
            // *** Die VIC II Sprite Register  -  ANFANG                                   ***
            // *******************************************************************************
            .var SPRITE0X            = $d000         // (00) X-Position von Sprite 0
            .var SPRITE0Y            = $d001         // (01) Y-Position von Sprite 0
            .var SPRITE1X            = $d002         // (02) X-Position von Sprite 1
            .var SPRITE1Y            = $d003         // (03) Y-Position von Sprite 1
            .var SPRITE2X            = $d004         // (04) X-Position von Sprite 2
            .var SPRITE2Y            = $d005         // (05) Y-Position von Sprite 2
            .var SPRITE3X            = $d006         // (06) X-Position von Sprite 3
            .var SPRITE3Y            = $d007         // (07) Y-Position von Sprite 3
            .var SPRITE4X            = $d008         // (08) X-Position von Sprite 4
            .var SPRITE4Y            = $d009         // (09) Y-Position von Sprite 4
            .var SPRITE5X            = $d00a         // (10) X-Position von Sprite 5
            .var SPRITE5Y            = $d00b         // (11) Y-Position von Sprite 5
            .var SPRITE6X            = $d00c         // (12) X-Position von Sprite 6
            .var SPRITE6Y            = $d00d         // (13) Y-Position von Sprite 6
            .var SPRITE7X            = $d00e         // (14) X-Position von Sprite 7
            .var SPRITE7Y            = $d00f         // (15) Y-Position von Sprite 7
            .var SPRITESMAXX         = $d010         // (16) Höhstes BIT der jeweiligen X-Position
                                                //         da der BS 320 Punkte breit ist reicht
                                                //         ein BYTE für die X-Position nicht aus// 
                                                //         Daher wird hier das 9. Bit der X-Pos
                                                //         gespeichert. BIT-Nr. (0-7) = Sprite-Nr.
            .var SPRITEACTIVE        = $d015         // (21) Bestimmt welche Sprites sichtbar sind
                                                //         Bit-Nr. = Sprite-Nr.
            .var SPRITEDOUBLEHEIGHT  = $d017         // (23) Doppelte Höhe der Sprites
                                                //         Bit-Nr. = Sprite-Nr.
            .var SPRITEDEEP          = $d01B         // (27) Legt fest ob ein Sprite vor oder hinter
                                                //         dem Hintergrund erscheinen soll.
                                                //         Bit = 1: Hintergrund vor dem Sprite
                                                //         Bit-Nr. = Sprite-Nr.
            .var SPRITEMULTICOLOR    = $d01c         // (28) Bit = 1: MultiColor Sprite 
                                                //         Bit-Nr. = Sprite-Nr.
            .var SPRITEDOUBLEWIDTH   = $d01d         // (29) Bit = 1: Doppelte Breite des Sprites
                                                //         Bit-Nr. = Sprite-Nr.
            .var SPRITEMULTICOLOR0   = $d025         // (37) Spritefarbe 0 im Multicolormodus
            .var SPRITEMULTICOLOR1   = $d026         // (38) Spritefarbe 1 im Multicolormodus
            .var SPRITE0COLOR        = $d027         // (39) Farbe von Sprite 0
            .var SPRITE1COLOR        = $d028         // (40) Farbe von Sprite 1
            .var SPRITE2COLOR        = $d029         // (41) Farbe von Sprite 2
            .var SPRITE3COLOR        = $d02a         // (42) Farbe von Sprite 3
            .var SPRITE4COLOR        = $d02b         // (43) Farbe von Sprite 4
            .var SPRITE5COLOR        = $d02c         // (44) Farbe von Sprite 5
            .var SPRITE6COLOR        = $d02d         // (45) Farbe von Sprite 6
            .var SPRITE7COLOR        = $d02e         // (46) Farbe von Sprite 7

            .var SPRITE0DATA         = $07f8         // Sprite-Pointer für die Sprite-0-Daten
            // *******************************************************************************
            // *** Die VIC II Sprite Register  -  ENDE                                     ***
            // *******************************************************************************

            // zone VIC
            .var VICBASE             = $d000         // (RG) = Register-Nr. dezimal
            .var VICRASTERROWPOS     = $d012         // (18) Aktuelle Rasterzeile lesen oder setzen
            .var VICBORDERCOLOR      = $d020         // (32) Bildschirmrandfarbe
            .var VICBACKGROUNDCOLOR  = $d021         // (33) Hintergrundfarbe

            // zone CIA
            .var CIA1_A              = $dc00         // Adresse des CIA1-A
            .var CIA1_B              = $dc01         // Adresse des CIA1-B
            .var CIA1_CLOCK_TENTH    = $dc08         // Adresse zehntel Sekunden (Uhrzeit) im CIA1
            .var CIA1_CLOCK_SECONDS  = $dc09         // Adresse Sekunden (Uhrzeit) im CIA1

            // zone SID
            // *******************************************************************************
            // *** Die SID Register  -  ANFANG                                             ***
            // *******************************************************************************
            .var SID_BASE            = $d400         // (Nr) Basisadresse des SID_BASE
            .var SID_VOICE1_FREQ_MSB = $d401         // (01) 1. Stimme: High-Byte der Frequenz
            .var SID_VOICE1_WAVEFORM = $d404         // (04) 1. Stimme: Wellenform
            .var SID_SUSTAIN_RELEASE = $d406         // (06) BIT 7-4: Anschlag (sustain)
                                                //          3-0: Abschwellen (release)
            .var SID_VOLUME          = $d418         // (24) BIT   7: 3. Stimme Filtermodus AUS
                                                //            6:   -''-    Hochpass
                                                //            5:   -''-    Bandpass
                                                //            4:   -''-    Tiefpass
                                                //          3-0: Lautstärke

            .var SID_WAVE_TRIANGLE   = $11           // Wellenform: Dreieck
            .var SID_WAVE_SAWTOOTH   = $21           // Wellenform: Sägezahn
            .var SID_WAVE_PULSE      = $41           // Wellenform: Rechteck
            .var SID_WAVE_WHITENOISE = $81           // Wellenform: Rauschen
            // *******************************************************************************
            // *** Die SID Register  -  ENDE                                               ***
            // *******************************************************************************

            // zone Zeropage
            .var ZP_SCREENRAMADR     = $02           // Zeichen-Quelle fürs BS-Kopieren (drawscreen:)
            .var ZP_COLORRAMADR      = $04           // Farb-Quelle fürs BS-Kopieren    (drawscreen:)
            .var ZP_SCREENTODRAW     = $06           // Zeichen-Ziel fürs BS-Kopieren   (drawscreen:)
            .var ZP_COLORTODRAW      = $08           // Farb-Ziel fürs BS-Kopieren      (drawscreen:)
            .var ZP_LOOPHELPER       = $f8           // Die letzten 8 BYTES für Schleifen reservieren

            // zone Kernal
            .var JT_GETIN            = $ffe4         // Kernalfunktion: Zeichen lesen

            // zone Generic
            .var SCREENRAMADR        = $0400         // Beginn des Bildschirmspeichers
            .var COLORRAMADR         = $d800         // Beginn des Farbspeichers
            .var INPUT_NONE          = $00           // Es wurde noch kein Port gewählt
            .var INPUT_JOY1          = $01           // Joystick in Port-1
            .var INPUT_JOY2          = $02           // oder 2
            .var JOY_UP              = %00000001     // Joystick rauf
            .var JOY_DOWN            = %00000010     // Joystick runter
            .var JOY_LEFT            = %00000100     // Joystick links
            .var JOY_RIGHT           = %00001000     // Joystick rechts
            .var JOY_FIRE            = %00010000     // Joystick FEUER// 



// --------------------------------------------
// LOAD DATA
// --------------------------------------------

// Music (<3k!)
// Attention: Reload Emulators to test other music!!

.var music = LoadSid("music7.sid") 

// Graphics
// https://mcdraw.xyz
//.if (loadPicture) {
  
  
    // .var picture = LoadBinary("image.koa", BF_KOALA)
    // .var picture = LoadBinary("minimalpicture.kla", BF_KOALA) // less loading
    .var picture = LoadBinary("white.kla", BF_KOALA)

//}

// --------------------------------------------------
// INIT
// --------------------------------------------------

// *=$0801 "Basic Program"
BasicUpstart2(start)


 // the address $0001 holds and sets the 6510 CPU's on-chip port register
    // %x10: RAM visible at $A000-$BFFF; KERNAL ROM visible at $E000-$FFFF
    // what we're saying here is we don't need the BASIC ROM after we start
    // see https://www.c64-wiki.com/wiki/Zeropage
    lda #%00110110
    sta $0001

// --------------------------------------------------
// INIT 
// --------------------------------------------------

start:

    // MULTIMEDIA

    // music: init 1
    .if (loadMusic) { 
        lda #music.startSong-1 // set up the music
        jsr music.init
    }

    // setup: raster music
    sei

        .if (loadMusic) { 
          lda #<irq1
          sta $0314
          lda #>irq1
          sta $0315
          lda #$1b
          sta $d011
          lda #$80
          sta $d012
          lda #$7f
          sta $dc0d
          sta $dd0d
          lda #$81
          sta $d01a
          lda $dc0d
          lda $dd0d
          asl $d019
        }

      cli


    // CLEAR SCREEN
    jsr clear_screen

     // disable input buffers     
     jsr disable_inputbuffers



// ------------------------------------
// asciiloop (SHOWS 'ASCII' MODE)
// ------------------------------------
// of course: it is no ascii!!!!
// different format like zx81 etc
 
jmp noascii // disable 'ASCII'    

    // background, frame color
      lda #$04     
      sta BORDER_COLOR      
      lda #$07
      sta BACKGROUND_COLOR  
      // set FOREGROUND_COLOR (COLOR CAN BE DIFFERENT IN EACH LINE!)
      lda #$09
      sta FOREGROUND_COLOR


      // ----------------
      // ASCII / PRINT
      // ----------------
      
      // charset: https://www.c64-wiki.de/wiki/Zeichen

      // print - show a character
      lda #$d6                //  load character '_' into register A
      jsr $ffd2               //  print character to screen
      // print at // ... 
      // https://medium.com/@RetroAsmDev/c-64-part-4-ascii-art-a4319784d26
      
  

        
        // WRITE INTO SCREEN MEMORY

        // ASCII: 40x25 > 1000bytes > $0400-$07F7
        // - example: https://dustlayer.com/c64-coding-tutorials/2013/4/8/episode-2-2-writing-to-the-c64-screen
        
        // FILL ALL WITH CHARS

        // all the same
        ldx #$00
        ldy #$00
clear:    
         lda #$40    // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         lda #$04     // set foreground to black in Color Ram 
         sta $d800,x  
         sta $d900,x
         sta $da00,x
         sta $dae8,x
         inx           // increment X
         bne clear    


         /*
        // write direct in the screen
        ldx #$ff
    directscreen:  
        lda #$04 
        sta $0400,x
        dex
        bne directscreen
        */

        // just print one char somewhere
        lda #$d8
        ldx #$20
        sta $0400,x


asciiloop: 
        jsr vsync // wait for 'vsync'

        jsr SCNKEY
        jsr GETIN

        // SPACE TO FINISH
        cmp #$20
        bne noinfokey
            // lda #05     
            // sta BORDER_COLOR      
            // sta BACKGROUND_COLOR  
            
            lda #$d6                //  load character '_' into register A
            jsr $ffd2               //  print character to screen
              
            jmp noascii
noinfokey:

        // PRINTING 
        // https://dustlayer.com/c64-coding-tutorials/2013/4/8/episode-2-4-reading-from-a-data-table

        ldx #$00
        ldy #$00

        ldy counter
        inc counter

        lda #$00        
clearcolhere:    
         lda #$42     // #$20 is the spacebar Screen Code
         sta $0400,x  // fill four areas with 256 spacebar characters
         sta $0500,x 
         sta $0600,x 
         sta $06e8,x 
         // lda #$08     // set foreground to black in Color Ram 
         iny
         tya
         sta $d800,x  
         iny
         tya
         sta $d900,x
         iny
         tya
         sta $da00,x
         // iny
         // tya
         sta $dae8,x
         inx           // increment X
         bne clearcolhere   

        // ------------------
        // blink  
        // ------------------
    
        /*
        inc BACKGROUND_COLOR
        inc BORDER_COLOR
        inc FOREGROUND_COLOR
        */

        jmp asciiloop

noascii:
        


      // ----------------
      // GRAPHIC MODE
      // ----------------

    //.if (loadPicture) {

// vicbank(%00000011);

vicbank(%00000010); // bank1


.if (true) {
      // copy picture 

        lda #$38
        sta $d018
        lda #$d8
        sta $d016
        lda #$3b
        sta $d011
        lda #0
        sta $d020
        lda #picture.getBackgroundColor()
        sta $d021
        ldx #0
!loop:
        .for (var i=0; i<4; i++) {
           lda colorRam+i*$100,x
           sta $d800+i*$100,x
        }
        inx
        bne !loop-
        jmp *

        
}

        // }


 // -----------------------------
  // SPRITE SETUP
  // -----------------------------
 
    // ALL SINGLE COLOR
    lda #%00000000
    // lda #%11111111
    sta $d01c // all sprites are of type 'single color'
    // card: $D015/53269  acts like a "switch" that turns one of the eight sprites "on" or "off"
    // swith all on 
    lda #%11111111
    sta $d015 // all eight sprites on (all bits set)

    // cards: the location of the sprite pointers (see sprites.asm) follows that of the text screen plus 1016/$3F8
     // this will iterate the 8 sprite pointers, load their values (see sprites.asm) and set screen addresses with them
    // see https://www.c64-wiki.com/wiki/Sprite#Sprite_pointers




jsr $e544 

    onespriteloop:

        // SIMPLE SPRITE SHOW
        lda #$02
        sta SPRITE_0_COLOR
        // reset data
        ldx #$00    // init x
        lda #$00    // init a
    clean:
        sta SPRITE_0_DATA,x   // write 0 into sprite data at x
        inx                   // increment x
        cpx #$3F              // is x <= 63?
        bne clean             // if yes, goto clean

        // lda #$04 // sprite multicolor 1
        // sta $D025

        // set sprite position
        ldx #$64                  //  initial x position = 100
        ldy #$46                  //  initial y position =  70
        stx SPRITE_0_X_POSITION   //  move sprite 0 to x position
        sty SPRITE_0_Y_POSITION   //  move sprite 0 to y position




    jmp onespriteloop

    // do the loop
    //  card   // $D001 (sprite 0), $D003, $D005, $D007, $D009, $D00B, $D00D, $D00F (sprite 7)
    // see https://www.c64-wiki.com/wiki/Sprite#Sprite_locations


      // copy picture 

        lda #$38
        sta $d018
        lda #$d8
        sta $d016
        lda #$3b
        sta $d011
        lda #0
        sta $d020
        lda #picture.getBackgroundColor()
        sta $d021
        ldx #0
!loop:
        .for (var i=0; i<4; i++) {
           lda colorRam+i*$100,x
           sta $d800+i*$100,x
        }
        inx
        bne !loop-
        jmp *


    ldx #$7
spriteloop:
/*
    lda spritePtrs,x
    sta screenRam_1 + 1016,x
    // lda 0
    // this hides sprites on screen 2
    // sta screenRam_2 + 1016,x

    // loads a random number from 0 to $F into the accumulator
    rndGen(%1111);
    // $D027-$D02E sets each sprite color
    // see https://www.c64-wiki.com/wiki/Sprite
    sta $d027,x
*/
    dex
    bpl spriteloop

// --------------------------------------------------
// GAME-LOOP 
// --------------------------------------------------


    // set background
    lda #00   
    sta BORDER_COLOR      
    sta BACKGROUND_COLOR  

 


/*
    // write direct in the screen
    ldx #$08
directscreen:  
    lda #$00
    // sta $0400,x
    sta $d800,x
    
    dex
    bne directscreen
*/

loop:
  

  // -----------------------------
  // SPRITE HANDLES
  // -----------------------------
 
    // $D01C/53276 sprites can be high resolution or multicolor mode - bits at 0 sets them to high res
    // see https://www.c64-wiki.com/wiki/Sprite
    lda #%00000000
    sta $d01c // all sprites are of type 'single color'

    // $D015/53269  acts like a "switch" that turns one of the eight sprites "on" or "off"
    // see https://www.c64-wiki.com/wiki/Sprite
    lda #%11111111
    sta $d015 // all eight sprites on (all bits set)




  // -----------------------------
  // KEYS INPUT
  // -----------------------------

           // get keys
           jsr SCNKEY
           jsr GETIN

              
            // SPACE TO FINISH
            cmp #$20
            bne nokey
            // jmp exit
            lda #05     
            sta BORDER_COLOR      
            sta BACKGROUND_COLOR  
            rts  // exit ... 
nokey:  

            // ------------------
            // joystick  
            // ------------------
            // joystick : https://retro-programming.com/how-to-program-joystick-input-handling-on-a-c64/

            lda #$08                // mask left movement (8 == bit 4 == %0000 1000)
            bit $DC01               // bitwise "and" with joystick port 1
            bne joystick_notright   // if not active (==1), go to .input_up_check
                // lda #$32
                jsr $ffd2   // print screen ... 
                inc SPRITE_0_X_POSITION
                inc SPRITE_0_X_POSITION
joystick_notright:
            
            lda #$01              // mask left movement (8 == bit 4 == %0000 1000)
            bit $DC01               // bitwise "and" with joystick port 1
            bne joystick_notleft   // if not active (==1), go to .input_up_check
                lda #$33
                // jsr $ffd2   // print screen ... 
                dec SPRITE_0_X_POSITION
                dec SPRITE_0_X_POSITION
                // dec #(SPRITE_0_X_POSITION-1)
            
            inc BACKGROUND_COLOR
            inc BORDER_COLOR
            inc FOREGROUND_COLOR

joystick_notleft:

   



            // wait for vsync
            jsr vsync // wait for 'vsync'

            // restart
            jmp loop      // loopy loop
exit: 
            jsr clear_screen
            rts

            // back to BASIC

// --------------------------------------------------
// DATA LOADING 
// --------------------------------------------------

//.if (loadPicture) {

    // new screen?
    *=$0c00;            .fill picture.getScreenRamSize(), picture.getScreenRam(i)
    *=$1c00; colorRam:  .fill picture.getColorRamSize(), picture.getColorRam(i)
    *=$2000;            .fill picture.getBitmapSize(), picture.getBitmap(i)

// }

.if (loadMusic) { 
    * = music.location "Music"
    .fill music.size, music.getData(i)
}

// VARIABLES
// counter
counter:
  .byte $00

    // --------------------------------------------------
    // OBJECTS ... 
    // --------------------------------------------------


    object_num: .byte $08

    .var _x = 0
    .var _y = 0
    .var _w = 0
    .var _h = 0
    .var _s = 0
    .var _a = 0
    .var _st = 0
    .var _sts = 0
    .var _arg = 0


    objects: 
      //    2 x      , 2y     , sizex, sizey, stat
      //    x [0,1]   y [2,3]   width[4] , height[5], spr[6], anim[7], stat[8], statsub[9], arg[10]

      // object  
      .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

 

racket:
.byte $00,$00,$00,$00,$00,$00,$03,$70
.byte $00,$0d,$9c,$00,$36,$a4,$3c,$36
.byte $e4,$3c,$36,$e4,$00,$35,$a4,$00
.byte $3d,$94,$00,$0d,$5c,$00,$0f,$70
.byte $00,$03,$c0,$f0,$00,$03,$ac,$00
.byte $0d,$7b,$00,$0f,$7b,$00,$0f,$db
.byte $00,$03,$fc,$00,$00,$f0,$00,$00
.byte $00,$00,$00,$00,$00,$00,$00,$81  

// --------------------------------------------------
// SUBROUTINES 
// --------------------------------------------------
clear_screen:
    jsr $E544       
    rts

disable_inputbuffers:
    lda #1
    sta $0289  //  disable keyboard buffer
    lda #127
    sta $028a  // disable key repeat
    rts

vsync:
      lda #$FF    // load 255 into A
      cmp $D012    //look if current rasterline equals 255
      bne vsync     // if no, goto wait
      rts  

.macro vicbank(pattern) {
    lda $dd00
    and #%11111100
    ora #pattern
    sta $dd00
}

// --------------------------------------------------
// INTERRUPTS 
// --------------------------------------------------
irq1:
    asl $d019
    

    // inc $d020 - act. volume 
    jsr music.play 
    // dec $d020
    pla
    tay
    pla
    tax
    pla
    rti


// --------------------------------------------------
// MACROS 
// --------------------------------------------------
.macro rndGen(mask) {
    // see http://sta.c64.org/cbm64mem.html
    // first we read the current raster line (0-255)
    lda $d012
    // then we xor it with timer A (low byte)
    // see https://www.c64-wiki.com/wiki/CIA
    eor $dc04
    // then we subtract it with timer A (high byte)
    sbc $dc05
    // finally we mask it so we can have a number between 0 and bits^2
    and #mask
}




            * = $3c00 "Virtual data" virtual        // <- Data in a virtual block is not entered into memory 
screen:     .fill $0400, 0


