
// ****************************************
// GAME
// ****************************************

// GRAPHICS PAIN
// https://mcdraw.xyz

// https://github.com/celso/c64


// MUSIC
// playing SID music has 3 parts:
// 0. load music
// 1. you use use Kick Assembler's LoadSid to import the .sid file
// 2. you call music.init
// 3. you call music.play in every raster interrupt
// see http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html and the README.md


    // ---------------
    // MINIMAL
    // ---------------
    .var  BORDER_COLOR   = $D020
    .var  BACKGROUND_COLOR    = $D021
    .var  FOREGROUND_COLOR = $E536

    .var  SPRITE_0_X_POSITION = $D000
    .var  SPRITE_0_Y_POSITION = $D001
    .var  SPRITE_0_POINTER    = $0400 + $03F8 // Last 8 Bytes of Screen RAM
    .var  SPRITE_0_DATA       = $0340         // Block 13, 13*64=>832 => $0340
    .var SPRITE_UPPER_X      = $D010
    .var  SPRITE_0_COLOR      = $D027 

     .var SCNKEY     = $ff9f   // scan keyboard - kernal routine
      .var GETIN      = $ffe4   // read keyboard buffer - kernal routine


.var music = LoadSid("music7.sid")
// Graphics
// https://mcdraw.xyz
.var picture = LoadBinary("image.koa", BF_KOALA)


        *=$0801 "Basic Program"
        BasicUpstart($0810)

        *=$0810 "Program"

// --------------------------------------------------
// INIT 
// --------------------------------------------------

    
    // music: init 1
    lda #music.startSong-1 // set up the music
    jsr music.init

    // setup: raster music
    sei
      lda #<irq1
      sta $0314
      lda #>irq1
      sta $0315
      lda #$1b
      sta $d011
      lda #$80
      sta $d012
      lda #$7f
      sta $dc0d
      sta $dd0d
      lda #$81
      sta $d01a
      lda $dc0d
      lda $dd0d
      asl $d019
      cli


      // copy picture 

        lda #$38
        sta $d018
        lda #$d8
        sta $d016
        lda #$3b
        sta $d011
        lda #0
        sta $d020
        lda #picture.getBackgroundColor()
        sta $d021
        ldx #0
!loop:
        .for (var i=0; i<4; i++) {
           lda colorRam+i*$100,x
           sta $d800+i*$100,x
        }
        inx
        bne !loop-
        jmp *

// --------------------------------------------------
// LOOP 
// --------------------------------------------------

loop:
  
  

            jsr SCNKEY
            jsr GETIN


              
            // SPACE
            cmp #$20
            bne nokey
            // jmp exit

            lda #05     
            sta BORDER_COLOR      
            sta BACKGROUND_COLOR  



            rts  // exit ... 
nokey:  

   
 
            // restart
            jmp loop      // loopy loop

            // ------------------ 


exit: 
            jsr clear_screen

  // back to BASIC

// --------------------------------------------------
// DATA LOADING 
// --------------------------------------------------

*=$0c00;            .fill picture.getScreenRamSize(), picture.getScreenRam(i)
*=$1c00; colorRam:  .fill picture.getColorRamSize(), picture.getColorRam(i)
*=$2000;            .fill picture.getBitmapSize(), picture.getBitmap(i)


* = music.location "Music"
.fill music.size, music.getData(i)

// --------------------------------------------------
// SUBROUTINES 
// --------------------------------------------------
clear_screen:
    jsr $E544       
    rts

disable_inputbuffers:
    lda #1
    sta $0289  //  disable keyboard buffer
    lda #127
    sta $028a  // disable key repeat
    rts

vsync:
      lda #$FF    // load 255 into A
      cmp $D012    //look if current rasterline equals 255
      bne vsync     // if no, goto wait
      rts  


// --------------------------------------------------
// INTERRUPTS 
// --------------------------------------------------
irq1:
    asl $d019
    

    // inc $d020 - act. volume 
    jsr music.play 
    // dec $d020
    pla
    tay
    pla
    tax
    pla
    rti


// --------------------------------------------------
// MACROS 
// --------------------------------------------------
.macro rndGen(mask) {
    // see http://sta.c64.org/cbm64mem.html
    // first we read the current raster line (0-255)
    lda $d012
    // then we xor it with timer A (low byte)
    // see https://www.c64-wiki.com/wiki/CIA
    eor $dc04
    // then we subtract it with timer A (high byte)
    sbc $dc05
    // finally we mask it so we can have a number between 0 and bits^2
    and #mask
}




