
// ****************************************
// IMAGE
// ****************************************

// GRAPHICS PAIN
// https://mcdraw.xyz

// https://github.com/celso/c64
.var picture1 = LoadBinary("image.koa", BF_KOALA)
.var picture2 = LoadBinary("screen2.koa", BF_KOALA)

// VIC-II bank 0
* = $0c00 "ScreenRam_1"; screenRam_1: .fill picture1.getScreenRamSize(), picture1.getScreenRam(i)
* = $1c00 "ColorRam_1:"; colorRam_1: .fill picture1.getColorRamSize(), picture1.getColorRam(i)
* = $2000 "Bitmap_1"; bitMap_1: .fill picture1.getBitmapSize(), picture1.getBitmap(i)

// VIC-II bank 1
* = $4c00 "ScreenRam_2"; screenRam_2: .fill picture2.getScreenRamSize(), picture2.getScreenRam(i)
* = $6000 "Bitmap_2"; bitMap_2: .fill picture2.getBitmapSize(), picture2.getBitmap(i)
* = $7f40 "ColorRam_2:"; colorRam_2: .fill picture2.getColorRamSize(), picture2.getColorRam(i)


// MUSIC
.var music = LoadSid("music.sid")

*=$0801   //  Starting Address BASIC + 1 =&gt//  SYS 2049
 
      .byte $0C,$08,$40,$00,$9E,$20,$32,$30,$36,$32,$00,$00,$00 //  BASIC CODE: 1024 SYS 2062

 
    // ---------------
    // MINIMAL
    // ---------------
    .var  BORDER_COLOR   = $D020
    .var  BACKGROUND_COLOR    = $D021
    .var  FOREGROUND_COLOR = $E536

    .var  SPRITE_0_X_POSITION = $D000
    .var  SPRITE_0_Y_POSITION = $D001
    .var  SPRITE_0_POINTER    = $0400 + $03F8 // Last 8 Bytes of Screen RAM
    .var  SPRITE_0_DATA       = $0340         // Block 13, 13*64=>832 => $0340
    .var SPRITE_UPPER_X      = $D010
    .var  SPRITE_0_COLOR      = $D027 

     .var SCNKEY     = $ff9f   // scan keyboard - kernal routine
      .var GETIN      = $ffe4   // read keyboard buffer - kernal routine


    // --------------------------------------------------
    // MUSIC 1
    // --------------------------------------------------

    // playing SID music has 3 parts:
    // 1. you use use Kick Assembler's LoadSid to import the .sid file
    // 2. you call music.init
    // 3. you call music.play in every raster interrupt
    // see http://www.theweb.dk/KickAssembler/webhelp/content/ch12s03.html and the README.md
    lda #music.startSong-1 // set up the music
    jsr music.init


// --------------------------------------------------
// INIT 
// --------------------------------------------------

init:

          // CLEAR SCREEN
          jsr clear_screen
    

          // Bitmap Mode On
          lda #$3b
          sta $d011
          // MultiColor On 
          lda #$d8
          sta $d016


          // picture?
          // recolorMacro(colorRam_1, picture1.getBackgroundColor(), 7);

            // crop the bottom of the screens off to create space for the scroll message
            // 320 pixels ($ff + $40)
            ldx #$ff
        !fill:  lda #0
            sta bitMap_1 + picture1.getBitmapSize() - 640,x
            dex
            bne !fill-
            ldx #$40
        !fill:  lda #0
            sta bitMap_1 + picture1.getBitmapSize() - 384,x
            dex
            bne !fill-
            ldx #$ff
        !fill:  lda #0
            sta bitMap_2 + picture2.getBitmapSize() - 640,x
            dex
            bne !fill-
            ldx #$40
        !fill:  lda #0
            sta bitMap_2 + picture2.getBitmapSize() - 384,x
            dex
            bne !fill-


        // ---------------
        // MUSIC 2
        // ---------------
        // disable the interrupts
        sei
        // first interrupt will be irq1 at scan line 240
        setupirq(240, irq1);
        // interrupt control - enable all interrupts with $7b
        // see http://sta.c64.org/cbm64mem.html
        lda #%01111011
        sta $dc0d
        // interrupt control register - enable raster interrupt with $81
        // see http://sta.c64.org/cbm64mem.html
        lda #%10000001
        sta $d01a
        // enable interrupts now
        cli



// --------------------------------------------------
// LOOP 
// --------------------------------------------------

loop:
  
  

            jsr SCNKEY
            jsr GETIN

              
            // SPACE
            cmp #$20
            bne nokey
            // jmp exit

            lda #05     
            sta BORDER_COLOR      
            sta BACKGROUND_COLOR  

            rts  // exit ... 
nokey:  

 
            // restart
            jmp loop      // loopy loop



exit: 

  // back to BASIC




// --------------------------------------------------
// SUBROUTINES 
// --------------------------------------------------
clear_screen:
    jsr $E544       
    rts

disable_inputbuffers:
    lda #1
    sta $0289  //  disable keyboard buffer
    lda #127
    sta $028a  // disable key repeat
    rts

vsync:
      lda #$FF    // load 255 into A
      cmp $D012    //look if current rasterline equals 255
      bne vsync     // if no, goto wait
      rts  

// --------------------------------------------------
// MUSIC 
// --------------------------------------------------

.macro ack() {
    lda #%11111111
    sta $d019
}

.macro exitirq() {
    // pull accumulator from stack
    pla
    // transfer accumulator to index y
    tay
    pla
    // transfer accumulator to index x
    tax
    // restore the accumulator
    pla
}


.macro setupirq(line, irq) {
    lda #line
    // write: raster line to generate interrupt at (bits #0-#7).
    sta $d012
    lda #<irq
    // execution address of interrupt service routine (left)
    sta $0314
    lda #>irq
    // execution address of interrupt service routine (right)
    sta $0315
}

// ok, we're using two interrupts here
// the first, irq1, is triggered at scanline 240
// the second, irq2, is triggered at scanline 10
// at the end irq1 we set irq2 and vice-versa

// irq1
// we use irq1 (from scanline 240 onwards) to change the VIC-II to text mode, scroll the bottom text message,
// rewrite it with scroll_message(), and play the music.

irq1:
    ack();
    // keep scrolling the bottom line
    // jsr scroll_message
    // keep the music playing
    jsr music.play
    // jump to irq2 at line 10
    //setupirq(10, irq2);
    // over and out
    exitirq();
    rti



* = music.location "Music"
.fill music.size, music.getData(i)
 
splashscreen:

.macro rndGen(mask) {
    // see http://sta.c64.org/cbm64mem.html
    // first we read the current raster line (0-255)
    lda $d012
    // then we xor it with timer A (low byte)
    // see https://www.c64-wiki.com/wiki/CIA
    eor $dc04
    // then we subtract it with timer A (high byte)
    sbc $dc05
    // finally we mask it so we can have a number between 0 and bits^2
    and #mask
}




